-- Settings are now modified under Interface Addon panel in game.

-- Default options that are loaded the first time the mod is loaded in the game
CEPlayerBar_Options = {
	["ChannelingBar"] = {
		["R"] = 1.0,
		["G"] = 0.7,
		["B"] = 0.0,
		["O"] = 1.0
	},
	["CastingBar"] = {
		["R"] = 1.0,
		["G"] = 0.7,
		["B"] = 0.0,
		["O"] = 0.5
	},
	["CastingText"] = {
		["R"] = 1.0,
		["G"] = 0.82,
		["B"] = 0.0
	},
	["NormalExp"] = {
		["R"] = 1.0,
		["G"] = 1.0,
		["B"] = 0.2,
		["O"] = 1.0
	},
	["RestedExp"] = {
		["R"] = 0.1,
		["G"] = 0.5,
		["B"] = 1.0,
		["O"] = 1.0
	},
	["RestedFutureExp"] = {
		["R"] = 0.4,
		["G"] = 0.8,
		["B"] = 1.0,
		["O"] = 1.0
	},
	["DisplayTime"] = 1,
	["DisplayExp"] = 1,
	["DisplayCast"] = 1,
	["DisplayPetBar"] = 1,
	["ShowSpellName"] = 1,
	["FontSize"] = 8,
	["HoverPattern"] = "&remainingxp to go",
	["HoverRestPattern"] = "&remainingxp to go",
	["NoHoverPattern"] = "&playername",
	["PercentEnabled"] = nil,
	["PercentPattern"] = "&remainingxppercent%",
	["Version"] = 1
}

-------------------------------------------------------------------
-- Local variables

-- variable that all of the Interface settings attach to
CEPB = {}; -- global table

local didMenuLoad = false;
CEPB_OldPetTexture = "";

-- variables to store the original player name color
local playerOrigR, playerOrigG, playerOrigB = nil;

-- casting related variables
CEPB_PlayerCasting = 0;
CEPB_PlayerChanneling = 0;
local playerSpell, playerStartTime, playerEndTime, playerFont, playerOldFontSize, playerOtherStuff = nil;

-- so we can hide the vehicle casting bar, but not a pet one
local IsPlayerInVehicle = 0;
local PetFrameHijacked = 0;

function CEPlayerBar_CheckVehicleStatus(self)
	IsPlayerInVehicle = UnitInVehicle("player");
	if (IsPlayerInVehicle == nil) then
		IsPlayerInVehicle = 0;
	end
	if (PetFrameHijacked == 0 and IsPlayerInVehicle == 1) then
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_START");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_STOP");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
		PetCastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
		PetFrameHijacked = 1;

	elseif (PetFrameHijacked == 1 and IsPlayerInVehicle == 0) then
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
		PetCastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
		PetFrameHijacked = 0;
	end
end

--------------------------------------------------------------------------------
-- Table management functions

function GetPlayerVal(option, table)
	if (table) then
		return CEPlayerBar_Options[table][option];
	else
		return CEPlayerBar_Options[option];
	end
end

function SetPlayerVal(option, value, table)
	if (table) then
		CEPlayerBar_Options[table][option] = value;
	else
		CEPlayerBar_Options[option] = value;
	end
end

--------------------------------------------------------------------------------
-- State management functions

function CEPlayerBar_Load()
	if (GetPlayerVal("DisplayCast") == 1) then
		CEPlayerBar_RegisterPlayerCastEvents(_G["CEPlayerBar_Frame"]);
	else
		CEPlayerBar_UnregisterPlayerCastEvents(_G["CEPlayerBar_Frame"]);
	end
	CEPlayerBar:Show();
	CEPlayerBarExhaust:Show();
	CEPlayerBar_ShowXP();
end

function CEPlayerBar_Unload()
	CEPlayerBar_UnregisterPlayerCastEvents(_G["CEPlayerBar_Frame"]);
	CEPlayerBar:Hide();
	CEPlayerBarExhaust:Hide();
end

function CEPlayerBar_ManageCurrentState()
	local isCastEnabled = GetPlayerVal("DisplayCast");
	local isExpEnabled = GetPlayerVal("DisplayExp");
	if (isCastEnabled == 0 and isExpEnabled == 0) then
		CEPlayerBar_Unload();
	else
		CEPlayerBar_Load();
	end
end

--------------------------------------------------------------------------------
-- Load

function CEPlayerBar_OnLoad (self)
	playerOrigR, playerOrigG, playerOrigB = PlayerName:GetTextColor();
	playerFont, playerOldFontSize, playerOtherStuff = PlayerName:GetFont();

	CEPlayerBar_RegisterPlayerExpEvents(self);
	if (GetPlayerVal("DisplayCast") == 1) then
		CEPlayerBar_RegisterPlayerCastEvents(self);
	end
	CEPlayerBar_RegisterAddonHeader();
end

function CEPlayerBar_RegisterPlayerExpEvents(self)
	self:RegisterEvent("PLAYER_XP_UPDATE");
	self:RegisterEvent("PLAYER_LEVEL_UP");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UPDATE_EXHAUSTION");
	self:RegisterEvent("VARIABLES_LOADED");
end

function CEPlayerBar_RegisterPlayerCastEvents(self)
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_START");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_STOP");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	CastingBarFrame:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
	CEPB_PlayerCasting = 0;
end

function CEPlayerBar_UnregisterPlayerExpEvents(self)
	self:UnregisterEvent("PLAYER_XP_UPDATE");
	self:UnregisterEvent("PLAYER_LEVEL_UP");
	self:UnregisterEvent("PLAYER_ENTERING_WORLD");
	self:UnregisterEvent("UPDATE_EXHAUSTION");
	self:UnregisterEvent("VARIABLES_LOADED");
end

function CEPlayerBar_UnregisterPlayerCastEvents(self)
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");

	self:UnregisterEvent("UNIT_SPELLCAST_START");
	self:UnregisterEvent("UNIT_SPELLCAST_STOP");
	self:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
	CEPB_PlayerCasting = 0;
end

function CEPlayerBar_RegisterAddonHeader()
	local frame = CreateFrame("FRAME", nil);
	frame:SetScript("OnShow",function() CEPlayerBar_ShowMenu() end);
	frame.name = "CEPlayerBar";

	InterfaceOptions_AddCategory(frame);
	InterfaceOptionsFrame:SetMovable(true);
	local title = InterfaceOptionsFrame:CreateTitleRegion();
	title:ClearAllPoints();
	title:SetAllPoints(InterfaceOptionsFrameHeader);
end

function CEPlayerBar_ShowMenu()
	if not didMenuLoad then
		CEPB:MakeBlizzOptions();
		didMenuLoad = true;
	else
		CEPB:OptionsFrame_OnShow();
	end
end

--------------------------------------------------------------------------------
-- Event handling

function CEPlayerBar_OnEvent (self, event, ...)
	local isEligible = 0;
	local type, _, _, _, _, _, _, _, _ = ...;

	-- Check vehicle status
	if ( (IsPlayerInVehicle == 1 and type == "vehicle") or type == "player") then
		isEligible = 1;
	end

	if ( event == "UNIT_SPELLCAST_START" and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		-- Get spell information
		playerSpell, _, _, _, playerStartTime, playerEndTime = UnitCastingInfo(type);
		playerStartTime = playerStartTime / 1000;
		playerEndTime = playerEndTime / 1000;

		-- Set cast bar properties
		CEPlayerBar:SetStatusBarColor(GetPlayerVal("R", "CastingBar"), GetPlayerVal("G", "CastingBar"), GetPlayerVal("B", "CastingBar"), GetPlayerVal("O", "CastingBar"));
		CEPlayerBar:SetMinMaxValues(playerStartTime, playerEndTime);
		CEPlayerBar:SetValue(playerStartTime);

		-- Set spell text properties
		PlayerName:SetFont(playerFont, GetPlayerVal("FontSize"), playerOtherStuff);
		PlayerName:SetTextColor(GetPlayerVal("R", "CastingText"), GetPlayerVal("G", "CastingText"), GetPlayerVal("B", "CastingText"));

		-- Show spell information
		if (GetPlayerVal("DisplayTime") == 1 and GetPlayerVal("ShowSpellName") == 1) then
			if (math.floor((playerEndTime - playerStartTime) / 1000) == (playerEndTime - playerStartTime) / 1000) then
				PlayerName:SetText( playerSpell .. " " .. ((playerEndTime - playerStartTime) / 1000) .. ".0");
			else
				PlayerName:SetText( playerSpell .. " " .. (playerEndTime - playerStartTime) / 1000);
			end

		elseif (GetPlayerVal("DisplayTime") == 1 and GetPlayerVal("ShowSpellName") == 0) then
			if (math.floor((playerEndTime - playerStartTime) / 1000) == (playerEndTime - playerStartTime) / 1000) then
				PlayerName:SetText(((playerEndTime - playerStartTime) / 1000) .. ".0");
			else
				PlayerName:SetText((playerEndTime - playerStartTime) / 1000);
			end

		elseif (GetPlayerVal("DisplayTime") == 0 and GetPlayerVal("ShowSpellName") == 1) then
			PlayerName:SetText( playerSpell );
		end

		CEPB_PlayerCasting = 1;
		-- Set to green for kicks... but opacity to 0% so it isn't visible
		CEPlayerBarExhaust:SetStatusBarColor(0.4, 0.8, 0.1, 0.0);

	elseif ( event == "UNIT_SPELLCAST_STOP" and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		if (CEPB_PlayerCasting == 1) then
			CEPB_PlayerCasting = 0;
			PlayerName:SetFont(playerFont, playerOldFontSize, playerOtherStuff);
			CEPlayerBar_ShowXP();
			CEPlayerBar_RealUpdate();
		end

	elseif (event == "UNIT_SPELLCAST_CHANNEL_STOP" and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		if (CEPB_PlayerChanneling == 1) then
			CEPB_PlayerChanneling = 0;
			PlayerName:SetFont(playerFont, playerOldFontSize, playerOtherStuff);
			CEPlayerBar_ShowXP();
			CEPlayerBar_RealUpdate();
		end

	elseif ( (event == "UNIT_SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED") and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		CEPB_PlayerCasting = 0;
		CEPB_PlayerChanneling = 0;
		PlayerName:SetFont(playerFont, playerOldFontSize, playerOtherStuff);
		CEPlayerBar_ShowXP();
		CEPlayerBar_RealUpdate();

	elseif ( event == "UNIT_SPELLCAST_DELAYED"  and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		local _, _, _, _, _, newEndTime = UnitCastingInfo(type);
		if (newEndTime ~= nil) then
			newEndTime = newEndTime / 1000;
			playerEndTime = newEndTime;
			CEPlayerBar:SetMinMaxValues(playerStartTime, playerEndTime);

			if (GetPlayerVal("DisplayTime") == 1) then
				if (math.floor(math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10) == math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10) then
					PlayerName:SetText((math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10) .. ".0");
				else
					PlayerName:SetText(math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10);
				end
			end
		end

	elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		-- Get spell information
		playerSpell, _, _, _, playerStartTime, playerEndTime = UnitChannelInfo(type);
		playerStartTime = playerStartTime / 1000;
		playerEndTime = playerEndTime / 1000;

		-- Set cast bar properties
		CEPlayerBar:SetMinMaxValues(playerStartTime, playerEndTime);
		CEPlayerBar:SetValue(playerEndTime);
		CEPlayerBar:SetStatusBarColor(GetPlayerVal("R", "ChannelingBar"), GetPlayerVal("G", "ChannelingBar"), GetPlayerVal("B", "ChannelingBar"), GetPlayerVal("O", "ChannelingBar"));

		-- Set spell text properties
		PlayerName:SetFont(playerFont, GetPlayerVal("FontSize"), playerOtherStuff);
		PlayerName:SetTextColor(GetPlayerVal("R", "CastingText"), GetPlayerVal("G", "CastingText"), GetPlayerVal("B", "CastingText"));

		-- Show spell information
		if (GetPlayerVal("DisplayTime") == 1 and GetPlayerVal("ShowSpellName") == 1) then
			if (math.floor((playerEndTime - playerStartTime) / 1000) == (playerEndTime - playerStartTime) / 1000) then
				PlayerName:SetText( playerSpell .. " " .. ((playerEndTime - playerStartTime) / 1000) .. ".0");
			else
				PlayerName:SetText( playerSpell .. " " .. (playerEndTime - playerStartTime) / 1000);
			end

		elseif (GetPlayerVal("DisplayTime") == 1 and GetPlayerVal("ShowSpellName") == 0) then
			if (math.floor((playerEndTime - playerStartTime) / 1000) == (playerEndTime - playerStartTime) / 1000) then
				PlayerName:SetText(((playerEndTime - playerStartTime) / 1000) .. ".0");
			else
				PlayerName:SetText((playerEndTime - playerStartTime) / 1000);
			end

		elseif (GetPlayerVal("DisplayTime") == 0 and GetPlayerVal("ShowSpellName") == 1) then
			PlayerName:SetText( playerSpell );
		end

		CEPB_PlayerCasting = 0;
		CEPB_PlayerChanneling = 1;

		-- Set to green for kicks... but opacity to 0% so it isn't visible
		CEPlayerBarExhaust:SetStatusBarColor(0.4, 0.8, 0.1, 0.0);

	elseif ( event == "UNIT_SPELLCAST_CHANNEL_UPDATE" and isEligible == 1 and GetPlayerVal("DisplayCast") == 1 ) then
		local _, _, _, _, _, newEndTime = UnitChannelInfo(type);
		if (newEndTime ~= nil) then
			newEndTime = newEndTime / 1000;
			if ( GetTime() >= newEndTime ) then
				CEPB_PlayerChanneling = 0;
				CEPlayerBar_ShowXP();
				CEPlayerBar_RealUpdate();

			elseif ( CEPB_PlayerChanneling == 1 ) then
				local durationShift = playerEndTime - newEndTime;
				playerEndTime = newEndTime;
				CEPlayerBar:SetValue( CEPlayerBar:GetValue() - durationShift );
				if (GetPlayerVal("DisplayTime") == 1) then
					if (math.floor(math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10) == math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10) then
						PlayerName:SetText( playerSpell .. " " .. (math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10) .. ".0");
					else
						PlayerName:SetText( playerSpell .. " " .. math.floor((CEPlayerBar:GetValue() * 10) - (playerStartTime * 10))/10);
					end
				end
			end
		end

	elseif ((event == "PLAYER_XP_UPDATE" or event == "PLAYER_LEVEL_UP" or event == "UPDATE_EXHAUSTION" or event == "PLAYER_ENTERING_WORLD") and CEPB_PlayerCasting ~= 1 and CEPB_PlayerChanneling ~= 1) then
		CEPlayerBar_ShowXP();
		CEPlayerBar_RealUpdate();
	end
end

function CEPlayerBar_ProcessBarSettingsWhileCasting()
	if (CEPB_PlayerChanneling == 1) then
		CEPlayerBar:SetStatusBarColor(GetPlayerVal("R", "ChannelingBar"), GetPlayerVal("G", "ChannelingBar"), GetPlayerVal("B", "ChannelingBar"), GetPlayerVal("O", "ChannelingBar"));
		PlayerName:SetTextColor(GetPlayerVal("R", "CastingText"), GetPlayerVal("G", "CastingText"), GetPlayerVal("B", "CastingText"));
	elseif (CEPB_PlayerCasting == 1) then
		CEPlayerBar:SetStatusBarColor(GetPlayerVal("R", "CastingBar"), GetPlayerVal("G", "CastingBar"), GetPlayerVal("B", "CastingBar"), GetPlayerVal("O", "CastingBar"));
		PlayerName:SetTextColor(GetPlayerVal("R", "CastingText"), GetPlayerVal("G", "CastingText"), GetPlayerVal("B", "CastingText"));
	end
end

--------------------------------------------------------------------------------
-- Handle clicking on new bar (extend player bar basically)

function CEPlayerBar_OnClick(self, button)
	if ( SpellIsTargeting() and button == "RightButton" ) then
		SpellStopTargeting();
		return;
	end

	if ( button == "LeftButton" ) then
		if ( SpellIsTargeting() ) then
			-- SpellTargetUnit("player");
		elseif ( CursorHasItem() ) then
			AutoEquipCursorItem();
		else
			-- TargetUnit("player");
		end
	else
		ToggleDropDownMenu(1, nil, PlayerFrameDropDown, "PlayerFrame", 106, 27);
		return;
	end
end

--------------------------------------------------------------------------------
-- Tooltip

function CEPlayerBar_ProcessToolTip()
	if (CEPlayerBar_IsHovering == 1 and (CEPB_PlayerChanneling == 1 or CEPB_PlayerCasting == 1)) then
		GameTooltip:SetOwner(PlayerFrame, "ANCHOR_CURSOR");
		GameTooltip:ClearLines();
		GameTooltip:AddLine(PlayerName:GetText());
		GameTooltip:Show();
	end
end

--------------------------------------------------------------------------------
-- Bar State functions

function CEPlayerBar_Update(self)
	CEPlayerBar_RealUpdate();

	if (CEPlayerBar_IsHovering == 0) then
		GameTooltip:Hide();
	end
end

function CEPlayerBar_RealUpdate()
	if (CEPB_PlayerCasting ~= 1) then
		if (CEPlayerBar_IsHovering == 1) then
			if (GetRestState() == 1) then
				PlayerName:SetText(CEPlayerBar_Process(GetPlayerVal("HoverRestPattern")));
			elseif (GetRestState() == 2) then
				PlayerName:SetText(CEPlayerBar_Process(GetPlayerVal("HoverPattern")));
			end
		else
			local tempVal = UnitInVehicle("player");
			if (tempVal == nil) then
				tempVal = 0;
			end

			if (tempVal == 1) then
				PlayerName:SetText(UnitName("vehicle"));
			else
				PlayerName:SetText(CEPlayerBar_Process(GetPlayerVal("NoHoverPattern")));
			end
		end

		if (GetPlayerVal("PercentEnabled") == 1) then
			CEPlayerBarPercent:Show();
			CEPlayerBarPercent:SetText(CEPlayerBar_Process(GetPlayerVal("PercentPattern")));
		else
			CEPlayerBarPercent:Hide();
		end

		if (GetPlayerVal("DisplayTime") == 1) then
			PlayerName:SetTextColor(playerOrigR, playerOrigG, playerOrigB);
		end
	end
end

function CEPlayerBar_ShowXP()
	CEPlayerBar:SetMinMaxValues(min(0, UnitXP("player")), UnitXPMax("player"));
	CEPlayerBarExhaust:SetMinMaxValues(min(0, UnitXP("player")), UnitXPMax("player"));
	if (GetPlayerVal("DisplayExp") == 1) then
		CEPlayerBar:SetValue(UnitXP("player"));
		local restedExp;
		if (GetXPExhaustion() ~= nil) then
			restedExp = max(((UnitXP("player") + GetXPExhaustion()) / UnitXPMax("player")) * CEPlayerBar:GetWidth(), 0);
		else
			restedExp = max(UnitXP("player") / UnitXPMax("player") * CEPlayerBar:GetWidth(), 0);
		end

		if (restedExp > CEPlayerBar:GetWidth()) then
			CEPlayerBarExhaust:SetValue(UnitXPMax("player"));
		else
			CEPlayerBarExhaust:SetValue(restedExp / CEPlayerBar:GetWidth() * UnitXPMax("player"));
		end

		if (GetRestState() ~= nil) then
			if (GetRestState() == 1) then
				CEPlayerBar:SetStatusBarColor(GetPlayerVal("R", "RestedExp"), GetPlayerVal("G", "RestedExp"), GetPlayerVal("B", "RestedExp"), GetPlayerVal("O", "RestedExp"));
				CEPlayerBarExhaust:SetStatusBarColor(GetPlayerVal("R", "RestedFutureExp"), GetPlayerVal("G", "RestedFutureExp"), GetPlayerVal("B", "RestedFutureExp"), GetPlayerVal("O", "RestedFutureExp")); 
			elseif (GetRestState() == 2) then
				CEPlayerBar:SetStatusBarColor(GetPlayerVal("R", "NormalExp"), GetPlayerVal("G", "NormalExp"), GetPlayerVal("B", "NormalExp"), GetPlayerVal("O", "NormalExp"));
			end
		end
	else
		CEPlayerBar:SetValue(0);
		CEPlayerBarExhaust:SetValue(0);
	end
end

function CEPlayerBar_UpdateTimers()
	if (CEPB_PlayerCasting == 1 or CEPB_PlayerChanneling == 1) then
		local time = GetTime();
		if ( time  > playerEndTime ) then
			time  = playerEndTime;
		end

		if ( CEPB_PlayerCasting == 1 ) then
			CEPlayerBar:SetValue(time);
		elseif ( CEPB_PlayerChanneling == 1 ) then
			local barValue = playerStartTime + (playerEndTime - time);
			CEPlayerBar:SetValue(barValue);
		end

		if (GetPlayerVal("DisplayTime") == 1 and GetPlayerVal("ShowSpellName") == 1) then
			if (math.floor(math.floor((playerEndTime * 10) - (time * 10))/10) == math.floor((playerEndTime * 10) - (time * 10))/10) then
				PlayerName:SetText( playerSpell .. " " .. (math.floor((playerEndTime * 10) - (time * 10))/10)..".0");
			else
				PlayerName:SetText( playerSpell .. " " .. math.floor((playerEndTime * 10) - (time * 10))/10);
			end

		elseif (GetPlayerVal("DisplayTime") == 1 and GetPlayerVal("ShowSpellName") == 0) then
			if (math.floor(math.floor((playerEndTime * 10) - (time * 10))/10) == math.floor((playerEndTime * 10) - (time * 10))/10) then
				PlayerName:SetText((math.floor((playerEndTime * 10) - (time * 10))/10)..".0");
			else
				PlayerName:SetText(math.floor((playerEndTime * 10) - (time * 10))/10);
			end

		elseif (GetPlayerVal("DisplayTime") == 0 and GetPlayerVal("ShowSpellName") == 1) then
			PlayerName:SetText( playerSpell );
		end

	end

	CEPlayerBar_ProcessToolTip();
end

--------------------------------------------------------------------------------
-- Process variables

function CEPlayerBar_Process(val)
	local temp;
	val = gsub(val, "&playername", UnitName("player"));
	val = gsub(val, "&percentxp", floor(UnitXP("player")/UnitXPMax("player")*100));
	val = gsub(val, "&remainingxppercent", floor((UnitXPMax("player") - UnitXP("player"))/UnitXPMax("player")*100));
	val = gsub(val, "&currentxp", UnitXP("player"));
	val = gsub(val, "&maxxp", UnitXPMax("player"));
	val = gsub(val, "&remainingxp", UnitXPMax("player")-UnitXP("player"));
	if (GetXPExhaustion() ~= nil) then
		temp = GetXPExhaustion()/2;
	else
		temp = "0"; 
	end
	val = gsub(val, "&restxp", temp);
	return val;
end